package bin;

import lib.CLIAppBase;
import kernel.peripherals.Peripherals.Peripheral;

using tink.CoreApi;
using Lambda;

class Disk extends CLIAppBase {
	public function new() {
		registerSyncSubcommand("ls", (args)->{
			Peripheral.instance.getDrives().foreach(drive -> {
				var addr = drive.getAddr();
				var label = drive.getDiskLabel();
				var id = drive.getDiskID();

				if (drive.isDiskPresent()){
					if (drive.hasAudio()){
						handle.writeLine('${addr} => ${label} [AUDIO]');
					}else{
						handle.writeLine('${addr} => ${label} (${id})');
					}
				}else {
					handle.writeLine('${addr} => [NO DISK]');
				}

				return true;
			});
		});

		registerSyncSubcommand("play", (args)->{
			if (args.length < 1){
				handle.writeLine("Missing drive address");
				return false;
			}

			return audioDiskPlayPause(args[0], true);
		},"<drive>");

		registerSyncSubcommand("stop", (args) -> {
			if (args.length < 1){
				handle.writeLine("Missing drive address");
				return false;
			}

			return audioDiskPlayPause(args[0], false);
		});

		registerSyncSubcommand("eject", (args)->{
			if (args.length < 1){
				handle.writeLine("Missing drive address");
				return false;
			}

			var driveAddr = args[0];
			var drive = Peripheral.instance.getDrive(driveAddr);

			if (drive == null){
				handle.writeLine("Drive not found: " + driveAddr);
				return false;
			}

			if (!drive.isDiskPresent()){
				handle.writeLine("No disk in drive: " + driveAddr);
				return false;
			}

			drive.ejectDisk();
			return true;
		},"<drive>");

		registerSyncSubcommand("lable",(args) -> {
			if (args.length < 1){
				handle.writeLine("Missing drive address");
				return false;
			}

			var driveAddr = args[0];
			var drive = Peripheral.instance.getDrive(driveAddr);
			var label:String = args[1];

			if (drive == null){
				handle.writeLine("Drive not found: " + driveAddr);
			}

			if (!drive.isDiskPresent()){
				handle.writeLine("No disk in drive: " + driveAddr);
			}

			if (label == null || label == ""){
				handle.writeLine(drive.getDiskLabel());
			}else{
				var err  = drive.setDiskLabel(label);
				if (err != null){
					handle.writeLine("Failed to set lable");
					return false;
				}
			}

			return true;
		},"<drive> [label]");
	}

	private function audioDiskPlayPause(driveAddr: String, play: Bool): Bool {
		var drive = Peripheral.instance.getDrive(driveAddr);

		if (drive == null){
			handle.writeLine("Drive not found: " + driveAddr);
			return false;
		}

		if (!drive.isDiskPresent()){
			handle.writeLine("No disk in drive: " + driveAddr);
			return false;
		}

		if (!drive.hasAudio()){
			handle.writeLine("Disk in drive: " + driveAddr + " does not have audio");
			return false;
		}

		if (play){
			drive.playAudio();
		}else{
			drive.stopAudio();
		}

		return true;
	}
}
