package kernel.ui;

import lib.ui.rendere.UIEventDelegate;
import lib.Pos;
import lib.Color;
import kernel.ButtonType;
import lib.Vec.Vec2;
import kernel.ui.TermWriteable;

using tink.CoreApi;

/**
	The main object you interact with when writing anything to the screen.
**/
class WindowContext implements TermWriteable {
	private final writer:VirtualTermWriter;

	@:allow(kernel.ui.WindowManager) private var eventDelegate: Null<UIEventDelegate>;

	public var onClick(default, null):Signal<{button:ButtonType, pos:Pos}>;
	public var onKey(default, null):Signal<{keyCode:Int, isHeld:Bool}>;
	public var onKeyUp(default, null):Signal<Int>;
	public var onMouseDrag(default, null):Signal<{button:ButtonType, pos:Pos}>;
	public var onMouseScroll(default, null):Signal<{dir:Int, pos:Pos}>;
	public var onMouseUp(default, null):Signal<{button:ButtonType, pos:Pos}>;
	public var onPaste(default, null):Signal<String>;
	public var onChar(default, null):Signal<String>;

	@:allow(kernel.ui.WindowManager) private final onClickTrigger:SignalTrigger<{button:ButtonType, pos:Pos}>;
	@:allow(kernel.ui.WindowManager) private final onKeyTrigger:SignalTrigger<{keyCode:Int, isHeld:Bool}>;
	@:allow(kernel.ui.WindowManager) private final onKeyUpTrigger:SignalTrigger<Int>;
	@:allow(kernel.ui.WindowManager) private final onMouseDragTrigger:SignalTrigger<{button:ButtonType, pos:Pos}>;
	@:allow(kernel.ui.WindowManager) private final onMouseScrollTrigger:SignalTrigger<{dir:Int, pos:Pos}>;
	@:allow(kernel.ui.WindowManager) private final onMouseUpTrigger:SignalTrigger<{button:ButtonType, pos:Pos}>;
	@:allow(kernel.ui.WindowManager) private final onPasteTrigger:SignalTrigger<String>;
	@:allow(kernel.ui.WindowManager) private final onCharTrigger:SignalTrigger<String>;

	@:allow(kernel.ui.WindowManager)
	private function new(writer:VirtualTermWriter) {
		this.writer = writer;
		this.onResize = writer.onResize;

		this.onClickTrigger = Signal.trigger();
		this.onKeyTrigger = Signal.trigger();
		this.onKeyUpTrigger = Signal.trigger();
		this.onMouseDragTrigger = Signal.trigger();
		this.onMouseScrollTrigger = Signal.trigger();
		this.onMouseUpTrigger = Signal.trigger();
		this.onPasteTrigger = Signal.trigger();
		this.onCharTrigger = Signal.trigger();

		this.onClick = onClickTrigger.asSignal();
		this.onKey = onKeyTrigger.asSignal();
		this.onKeyUp = onKeyUpTrigger.asSignal();
		this.onMouseDrag = onMouseDragTrigger.asSignal();
		this.onMouseScroll = onMouseScrollTrigger.asSignal();
		this.onMouseUp = onMouseUpTrigger.asSignal();
		this.onPaste = onPasteTrigger.asSignal();
		this.onChar = onCharTrigger.asSignal();
	}

	public var onResize(default, null):Signal<Vec2<Int>>;

	@:allow(kernel.ui)
	private inline function setTarget(target:TermWriteable) {
		writer.setTarget(target);
	}

	@:allow(kernel.ui)
	private inline function enable() {
		writer.enable();
	}

	@:allow(kernel.ui)
	private inline function disable() {
		writer.disable();
	}

	@:allow(kernel.ui)
	private inline function isEnabled() {
		return writer.isEnabled();
	}

	public inline function write(text:String) {
		writer.write(text);
	}

	public inline function scroll(y:Int) {
		writer.scroll(y);
	}

	public inline function getCursorPos():Pos {
		return writer.getCursorPos();
	}

	public inline function setCursorPos(x:Int, y:Int) {
		writer.setCursorPos(x, y);
	}

	public inline function getCursorBlink():Bool {
		return writer.getCursorBlink();
	}

	public inline function setCursorBlink(blink:Bool) {
		writer.setCursorBlink(blink);
	}

	public inline function getSize():Vec2<Int> {
		return writer.getSize();
	}

	public inline function clear() {
		writer.clear();
	}

	public inline function clearLine() {
		writer.clearLine();
	}

	public inline function getTextColor():Color {
		return writer.getTextColor();
	}

	public inline function setTextColor(color:Color) {
		writer.setTextColor(color);
	}

	public inline function getBackgroundColor():Color {
		return writer.getBackgroundColor();
	}

	public inline function setBackgroundColor(color:Color) {
		writer.setBackgroundColor(color);
	}

	public inline function isColor():Bool {
		return writer.isColor();
	}

	public inline function reset() {
		writer.reset();
	}

	/**
		Delegate events to an UIEventDelegate.
		Set to null to stop delegating events.
	**/
	public inline function delegateEvents(delegate: Null<UIEventDelegate>){
		this.eventDelegate = delegate;
	}
}
